<?php
/**
 * @file
 * The rules implementation for the To Do module.
 */

/**
 * Implementation of hook_rules_event_info().
 * The To do events information.
 */
function to_do_rules_rules_event_info() {
  global $user;

  $params = array(
    'node' => array('type' => 'node', 'label' => 'to do task'),
    'assigned_user' => array('type' => 'user', 'label' => 'assigned user'),
    'author' => array('type' => 'user', 'label' => 'to do author', 'handler' => 'rules_events_argument_node_author'),
  );
  $params = $params + rules_events_global_user_argument();
  return array(
    'to_do_created' => array(
      'label' => t('To do event created'),
      'module' => 'To do tasks',
      'arguments' => $params,
      'help' => t('A user saved a new To do task.'),
    ),
    'to_do_updated' => array(
      'label' => t('To do event updated'),
      'module' => 'To do tasks',
      'arguments' => $params,
      'help' => t('A user updated a To do task.'),
    ),
    'to_do_started' => array(
      'label' => t('To do event started'),
      'module' => 'To do tasks',
      'arguments' => $params,
      'help' => t('A user clicked on the Started button.'),
    ),
    'to_do_reminder' => array(
      'label' => t('To do event reminder'),
      'module' => 'To do tasks',
      'arguments' => $params,
      'help' => t('CRON detected that a reminder should be sent.'),
    ),
    'to_do_deadline_reached' => array(
      'label' => t('To do event deadline reached'),
      'module' => 'To do tasks',
      'arguments' => $params,
      'help' => t('CRON detected that a deadline was reached.'),
    ),
    'to_do_past_deadline' => array(
      'label' => t('To do event past deadline'),
      'module' => 'To do tasks',
      'arguments' => $params,
      'help' => t('CRON detected that To do task is past its deadline.'),
    ),
    'to_do_finished' => array(
      'label' => t('To do event was completed'),
      'module' => 'To do tasks',
      'arguments' => $params,
      'help' => t('The current user clicked the Finished button of this To do task.'),
    ),
    'to_do_unpublished' => array(
      'label' => t('To do event was unpublished automatically'),
      'module' => 'To do tasks',
      'arguments' => $params,
      'help' => t('The system determined that this task was completed and could automatically be unpublished.'),
    ),
    'to_do_deleting' => array(
      'label' => t('To do event is being deleted automatically'),
      'module' => 'To do tasks',
      'arguments' => $params,
      'help' => t('The system determined that this task was completed a while back and could automatically be deleted.'),
    ),
  );
}

/**
 * Implementation of hook_rules_condition_info()
 * The To do condition information.
 */
function to_do_rules_rules_condition_info() {
  $items = array();
  $items['rules_condition_to_do_has_status'] = array(
    'label' => t('To do has status'),
    'arguments' => array(
      'item_status' => array('type' => 'string', 'label' => t('Item status')),
      'node' => array('type' => 'node', 'label' => t('Select the To do task')),
    ),
    'module' => 'To do rules',
    'help' => t('Check the current To do node status.'),
  );
  return $items;
}

/**
 * Condition: check whether the To do node has the specified status
 */
function rules_condition_to_do_has_status($values, $node, $settings) {
  $values = $settings['item_status'];
  if (is_array($values)) {
    foreach ($values as $key => $v) {
      // note that the value was increased by 1 in the form
      if ($v && $node->item_status == $v - 1) {
        return TRUE;
      }
    }
  }
  return FALSE;
}

// vim: ts=2 sw=2 et syntax=php
